<?php

/**
 * xt:Commerce callback class.
 *
 * Extends the callback
 *
 * @author GiroSolution AG
 * @link http://www.girosolution.de
 * @package GiroCheckout
 * @copyright Copyright (c) 2014, GiroSolution AG
 */
defined('_VALID_CALL') or die('Direct Access is not allowed.');

require_once(_SRV_WEBROOT . _SRV_WEB_PLUGINS . 'xt_girocheckout/classes/class.xt_girocheckout.php');
require_once(_SRV_WEBROOT . _SRV_WEB_PLUGINS . 'xt_girocheckout/classes/api/GiroCheckout_SDK.php');

class callback_xt_girocheckout extends callback {

  var $version = '6.0.0';
  var $lang = 'de';
  var $data = array();

  /**
   * Process the redirect and notify URL from the payment bank page.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @param string $subModule The payment method
   * @return none
   */
  function process() {
    global $xtLink, $filter, $_GET, $_POST, $language;

    // Parameter aus URL lesen.
    $action = $_REQUEST['act'];
    $pm = $_REQUEST['pm'];

    if (isset($_REQUEST['gcMerchantTxId'])) {
      $transactionId = $_REQUEST['gcMerchantTxId'];
    }

    if (!isset($action) || !isset($transactionId)) {
      // Benachrichtigung soll nochmals erfolgen.
      header('HTTP/1.1 503 Service Unavailable');
      echo 'input param action or transactionId missing';
      exit;
    }

    $this->data = $_REQUEST;
    if (!isset($this->data['transaction_id'])) {
      $this->data['transaction_id'] = $transactionId;
    }
    if (isset($language)) {
      if ($language->code == 'en') {
        $this->lang = 'en';
      }
    }

    $result = $this->_callbackProcess();
  }

  /**
   * Clear the order.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @param int $orderId The order id
   * @return none
   */
  function _clearOrder($orderId = null) {
    global $xtPlugin, $db;
    if ($orderId !== null) {
      $stock = new stock();
      $rs = $db->Execute("SELECT products_id,products_quantity FROM " . TABLE_ORDERS_PRODUCTS . " WHERE orders_id = '" . (int) $orderId . "'");
      while (!$rs->EOF) {
        $stock->addStock($rs->fields['products_id'], $rs->fields['products_quantity']);
        $rs->MoveNext();
      }

      $db->Execute("DELETE FROM " . TABLE_ORDERS . " WHERE orders_id = '" . (int) $orderId . "'");
      $db->Execute("DELETE FROM " . TABLE_ORDERS_PRODUCTS . " WHERE orders_id = '" . (int) $orderId . "'");
      $db->Execute("DELETE FROM " . TABLE_ORDERS_PRODUCTS_MEDIA . " WHERE orders_id = '" . (int) $orderId . "'");
      $db->Execute("DELETE FROM " . TABLE_ORDERS_TOTAL . " WHERE orders_id = '" . (int) $orderId . "'");
      $db->Execute("DELETE FROM " . TABLE_ORDERS_STATS . " WHERE orders_id = '" . (int) $orderId . "'");
      $db->Execute("DELETE FROM " . TABLE_ORDERS_STATUS_HISTORY . " WHERE orders_id = '" . (int) $orderId . "'");
    }
  }

  /**
   * Send the order by mail to customer.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @param object $order The order data
   * @param object $payment The payment data
   * @return bool
   */
  function _sendOrderMail($order = null, $payment = null) {
    global $xtPlugin, $db, $store_handler;

    ($plugin_code = $xtPlugin->PluginCode('class.orders.php:_sendOrderMail_top')) ? eval($plugin_code) : false;
    if ($order !== null) {

      $lang = 'de';
      if ($order->order_data['language_code'] == 'en') {
        $lang = $order->order_data['language_code'];
      }

      if ($payment !== null) {
        $order->order_data['payment_name'] = $payment;
      }

      $ordermail = new xtMailer('send_order', $order->order_data['language_code'], $order->order_customer['customers_status']);
      $ordermail->_addReceiver($order->order_data['customers_email_address'], $order->order_data['billing_lastname'] . ' ' . $order->order_data['billing_firstname']);
      $ordermail->_assign('order_customer', $order->order_customer);
      $ordermail->_assign('order_data', $order->order_data);
      $ordermail->_assign('order_products', $order->order_products);
      $ordermail->_assign('order_total_data', $order->order_total_data);
      $ordermail->_assign('total', $order->order_total);
      $ordermail->_assign('order_count', $order->order_count);

      // get text for payment method
      $rs = $db->Execute("SELECT pd.payment_email_desc_txt,pd.payment_email_desc_html FROM " . TABLE_PAYMENT_DESCRIPTION . " pd, " . TABLE_PAYMENT . " p WHERE pd.language_code='" . $order->order_data['language_code'] . "' and p.payment_id=pd.payment_id and p.payment_code='" . $order->order_data['payment_code'] . "'");
      if ($rs->RecordCount() == 1) {
        // old payment info
        $ordermail->_assign('payment_info', $rs->fields['payment_email_desc_txt']);

        // new payment info
        $ordermail->_assign('payment_info_html', $rs->fields['payment_email_desc_html']);
        $ordermail->_assign('payment_info_txt', $rs->fields['payment_email_desc_txt']);
      }

      ($plugin_code = $xtPlugin->PluginCode('class.orders.php:_sendOrderMail_bottom')) ? eval($plugin_code) : false;
      $ordermail->_sendMail();

      $q = "update " . TABLE_ORDERS_STATUS_HISTORY . "
            set customer_notified = '1'
            where orders_id = '" . $this->orders_id . "'
            order by orders_status_history_id desc
            limit 1";
      $db->Execute($q);

      return true;
    }

    return false;
  }

  /**
   * Process the callback.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return bool
   */
  function _callbackProcess() {
    global $xtLink, $_GET, $_POST;

    if ($this->log_callback_data == true) {
      $log_data = array();
      $log_data['module'] = 'xt_girocheckout';
      $log_data['class'] = 'callback_data';
      $log_data['transaction_id'] = $this->data['transaction_id'];
      $log_data['callback_data'] = serialize($this->data);
      $this->_addLogEntry($log_data);
    }
    
    $delete_order = false;
    $action = $this->data['act'];
    $pm = $this->data['pm'];
    
    $subModulePref = strtoupper('xt_girocheckout_' . $pm . '_');
    if (defined($subModulePref . 'SECURE')) {
      $secure = constant($subModulePref . 'SECURE');
    }

    switch ($pm) {
      case 'gc_giropay':
        $gcnotifytype = 'giropayTransaction';
        break;

      case 'gc_creditcard':
        $gcnotifytype = 'creditCardTransaction';
        break;

      case 'gc_directdebit':
        $gcnotifytype = 'directDebitTransactionWithPaymentPage';
        break;

      case 'gc_eps':
        $gcnotifytype = 'epsTransaction';
        break;

      case 'gc_ideal':
        $gcnotifytype = 'idealPayment';
        break;
      
      case 'gc_sofortuw':
        $gcnotifytype = 'sofortuwTransaction';
        break;
      
      case 'gc_paydirekt':
        $gcnotifytype = 'paydirektTransaction';
        break;

      default:
        header('HTTP/1.1 400 Bad Request');
        echo 'Invalid payment method.';
        exit;
        break;
    }

    // Get the notification
    $paymentSUCCESS = FALSE;

    try {
      $notify = new GiroCheckout_SDK_Notify($gcnotifytype);
      $notify->setSecret(trim((string) $secure));
      $notify->parseNotification($this->data);

      //Checks if the payment was successful and redirects the user
      if ($notify->paymentSuccessful()) {
        $paymentSUCCESS = true;
      }
    }
    catch (Exception $e) {
    }

    // order ID already inserted ?
    if (!$this->_getOrderID()) {
      if ($action == 'redirect') {
        $_SESSION['cart']->_resetCart();
        xt_girocheckout::clearSession();
        $link = $xtLink->_link(array('page' => 'cart'));
        $xtLink->_redirect($link);
        header('HTTP/1.1 200 OK');
      }
      else {
        $notify->sendOkStatus();
        $notify->setNotifyResponseParam('Result', 'OK');
        $notify->setNotifyResponseParam('ErrorMessage', '');
        $notify->setNotifyResponseParam('MailSent', '');
        $notify->setNotifyResponseParam('OrderId', $this->orders_id);
        $notify->setNotifyResponseParam('CustomerId', $this->customers_id);
        echo $notify->getNotifyResponseStringJson();
      }
      exit;
    }

    $iOrderId = $this->orders_id;
    $iCustomerId = $this->customers_id;

    
    if ($action == 'notify') {
      $this->notified = false;
      switch ($pm) {
        case 'gc_giropay':
        case 'gc_directdebit':
        case 'gc_creditcard':
        case 'gc_eps':
        case 'gc_ideal':
        case 'gc_sofortuw':
        case 'gc_paydirekt':
          
          $erMsg = $notify->getResponseMessage($notify->getResponseParam('gcResultPayment'),'DE');
          
          if( $pm == 'gc_giropay' ) {
            $iReturnCodeAVS = $notify->getResponseParam('gcResultAVS');
            if (!$notify->avsSuccessful() && $iReturnCodeAVS != "4900" && ($iReturnCodeAVS == "4020" || $iReturnCodeAVS == "4021" || $iReturnCodeAVS == "4022")) {
              $paymentAVSMsg = $notify->getResponseMessage($iReturnCodeAVS, 'DE');
              if (!empty($paymentAVSMsg)) {
                $erMsg .= " (" . $paymentAVSMsg . ")";
              }
            }
          }
          
          if( $paymentSUCCESS ) {
            if($this->_getStatusHistory()==0) {
              $this->data['comments'] = 'payment with ' . $pm . ' successful';
              $this->notified = true;
              $order = new order($this->orders_id, $this->customers_id); // set in _getOrderID()
              $this->data['status'] = 2; // success
              $this->_setStatus();
              $this->_sendOrderMail($order, $pm);
            }
            $notify->sendOkStatus();
            $notify->setNotifyResponseParam('Result', 'OK');
            $notify->setNotifyResponseParam('ErrorMessage', '');
            $notify->setNotifyResponseParam('MailSent', '');
            $notify->setNotifyResponseParam('OrderId', $iOrderId);
            $notify->setNotifyResponseParam('CustomerId', $iCustomerId);
            echo $notify->getNotifyResponseStringJson();
          }
          else {
            if($this->_getStatusHistory()==0) {
              $this->data['comments'] = $pm . ' payment error: ' . $erMsg;
              if ($delete_order) {
                $order = new order(); // set in _getOrderID()
                $order->_deleteOrder($this->orders_id); // bug? not delete all order info
                $this->_clearOrder($this->orders_id);
              } else {
                $this->data['status'] = -1; // error
                $this->_setStatus();
              }
            }
            $notify->sendOkStatus();
            $notify->setNotifyResponseParam('Result', 'ERROR');
            $notify->setNotifyResponseParam('ErrorMessage', $erMsg );
            $notify->setNotifyResponseParam('MailSent', '');
            $notify->setNotifyResponseParam('OrderId', $iOrderId);
            $notify->setNotifyResponseParam('CustomerId', $iCustomerId);
            echo $notify->getNotifyResponseStringJson();
          }
          break;
      }
      exit;
    }
    elseif ($action == 'redirect'){
      if ($paymentSUCCESS) {
        if($this->_getStatusHistory()==0) {
          $this->data['comments'] = 'payment with ' . $pm . ' successful';
          $this->notified = true;
          $order = new order($this->orders_id, $this->customers_id); // set in _getOrderID()
          $this->data['status'] = 2; // success
          $this->_setStatus();
          $this->_sendOrderMail($order, $pm);
        }
        unset($_SESSION['last_order_id']);
        unset($_SESSION['selected_shipping']);
        unset($_SESSION['selected_payment']);
        unset($_SESSION['conditions_accepted']);
        $_SESSION['cart']->_resetCart();
        xt_girocheckout::clearSession();

        $link = $xtLink->_link(array('page' => 'checkout',
            'paction' => 'success'
        ));
      }
      else {
        if($this->_getStatusHistory()==0) {
          $erMsg = $notify->getResponseMessage($notify->getResponseParam('gcResultPayment'), 'DE');
          $this->data['comments'] = $pm . ' payment error: ' . $erMsg;

          if ($delete_order) {
            $order = new order(); // set in _getOrderID()
            $order->_deleteOrder($this->orders_id); // bug? not delete all order info
            $this->_clearOrder($this->orders_id);
          } else {
            $this->data['status'] = -1; // error
            $this->_setStatus();
          }
        }

          $erMsg = GiroCheckout_SDK_ResponseCode_helper::getMessage($this->data['gcResultPayment'], $this->lang);
          xt_girocheckout::setErrorMsg($erMsg);
          $link = $xtLink->_link(array('page' => 'checkout',
              'paction' => 'payment',
              'params' => 'error=' . urlencode($erMsg),
              'conn' => 'SSL'
          ));

      }

      $xtLink->_redirect($link);
      exit;
    }

    return true;
  }

  /**
   * Get the order id.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return bool
   */
  function _getOrderID() {
    global $db;

    $order_query = "SELECT orders_id, customers_id FROM " . TABLE_ORDERS . " WHERE orders_id = '" . (int) $this->data['transaction_id'] . "'";
    $rs = $db->Execute($order_query);

    if ($rs->RecordCount() == 1) {
      $this->orders_id = $rs->fields['orders_id'];
      $this->customers_id = $rs->fields['customers_id'];

      return true;
    }

    $log_data = array();
    $log_data['module'] = 'xt_girocheckout';
    $log_data['class'] = 'error';
    $log_data['error_msg'] = 'order id not found';
    $log_data['error_data'] = serialize(array('transaction_id' => $this->data['transactionId']));
    $this->_addLogEntry($log_data);

    return false;
  }

  /**
   * Get Status History.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return int
   */
  function _getStatusHistory() {
    global $db;
    $order_query = "SELECT * FROM " . TABLE_ORDERS_STATUS_HISTORY . " WHERE orders_id = '" . (int) $this->data['transaction_id'] . "' AND change_trigger ='IPN'";
    $rs = $db->Execute($order_query);
    return $rs->RecordCount();
  }

  /**
   * Set the order status.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return none
   */
  function _setStatus() {

    global $db;

    $do_mail = 'false';
    switch ($this->data['status']) {

      case 2 :
        //$do_mail = 'true';
        $status = XT_GIROCHECKOUT_ORDER_STATUS_SUCCESS;
        $log_data = array();
        $log_data['orders_id'] = $this->orders_id;
        $log_data['module'] = 'xt_girocheckout';
        $log_data['class'] = 'success';
        $log_data['transaction_id'] = $this->orders_id;
        $log_data['callback_data'] = array('message' => 'OK',
            'error' => $this->data['returnCode'],
            'transaction_id' => $this->orders_id
        );
        $txn_log_id = $this->_addLogEntry($log_data);
        break;
      case 1 :
        //$do_mail = 'true';
        $status = XT_GIROCHECKOUT_ORDER_STATUS_ACCEPTED;
        $log_data = array();
        $log_data['orders_id'] = $this->orders_id;
        $log_data['module'] = 'xt_girocheckout';
        $log_data['class'] = 'success';
        $log_data['transaction_id'] = $this->orders_id;
        $log_data['callback_data'] = array('message' => 'OK',
            'error' => $this->data['returnCode'],
            'transaction_id' => $this->orders_id
        );
        $txn_log_id = $this->_addLogEntry($log_data);
        break;
      case 0 :
        break;
      case -1 :
        $status = XT_GIROCHECKOUT_ORDER_STATUS_ERROR;
        $log_data = array();
        $log_data['orders_id'] = $this->orders_id;
        $log_data['module'] = 'xt_girocheckout';
        $log_data['class'] = 'success';
        $log_data['transaction_id'] = $this->orders_id;
        $log_data['callback_data'] = array('message' => 'FAILED',
            'error' => $this->data['returnCode'],
            'transaction_id' => $this->orders_id
        );
        $txn_log_id = $this->_addLogEntry($log_data);
        break;
      case -2 :
      case -3:
        break;
    }
    // update order status
    $this->_updateOrderStatus($status, $do_mail, $txn_log_id);

    $q = "update " . TABLE_ORDERS_STATUS_HISTORY . "
            set comments = '" . $this->data['comments'] . "'
            where orders_id = '" . $this->orders_id . "'
                and callback_id = '" . $txn_log_id . "'";
    $db->Execute($q);
  }
} 