<?php

/**
 * xt:Commerce plugin class.
 *
 * Extends the payment
 *
 * @author GiroSolution AG
 * @link http://www.girosolution.de
 * @package GiroCheckout
 * @copyright Copyright (c) 2014, GiroSolution AG
 */
defined('_VALID_CALL') or die('Direct Access is not allowed.');
require_once(dirname(__FILE__) . '/api/GiroCheckout_SDK.php');

GiroCheckout_SDK_Config::getInstance()->setConfig('DEBUG_LOG_PATH',  _SRV_WEBROOT.'/xtLogs/girocheckout');
//GiroCheckout_SDK_Config::getInstance()->setConfig('DEBUG_MODE',1);

class xt_girocheckout extends payment {

  public $data = array();
  public $version = '6.0.0';
  public $external = true;
  public $iframe = false;
  public $post_form = false;
  public $subpayments = true;
  public $allowed_subpayments;
  public $lang = 'de';
  public $sourceID = 'c3ba6c99e342084767a311a8460db065';
  public $aTexts = array();
  public $sysversion4 = 'false';

  function __construct() {
    global $page, $xtLink, $language;

    $this->lang = 'de';
    if (isset($language)) {
      if ($language->code === 'en') {
        $this->lang = $language->code;
      }
    }

    $this->allowed_subpayments = array(
        'gc_sofortuw',
        'gc_giropay',
        'gc_directdebit',
        'gc_creditcard',
        'gc_ideal',
        'gc_eps',
        'gc_paydirekt',
    );

    $this->loadText();

    // Circumvent the selected payment sub name.
    foreach ($this->allowed_subpayments as $subModule) {
      $subDef = 'TEXT_PAYMENT_' . strtoupper($subModule);
      $subDefName = $this->aTexts[$this->lang][$subModule]['title'];
      if (!defined($subDef))
        define('TEXT_PAYMENT_' . strtoupper($subModule), $subDefName);
    }

    $this->getConfFieldsBySubModules();
    $this->getSubModulesInfo();
    $this->handlePageAction($page->page_name, $page->page_action);
    uasort($this->data, array( $this, "sortMethods" ));

    $this->data["aPayments"] = $this->data;

  }

  /**
   * Check if the payment method is allowed.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @param string $subModule The payment method
   * @return bool
   */
  function subModuleIsAllowed($subModule = '') {
    return in_array($subModule, $this->allowed_subpayments);
  }

  /**
   * Check if the payment method is activated.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @param string $subModule The payment method
   * @return bool
   */
  function subModuleIsActivated($subModule = '') {
    $subModulePref = strtoupper(get_class($this) . '_' . $subModule . '_');
    if (defined($subModulePref . 'ACTIVATED')) {
      if (constant($subModulePref . 'ACTIVATED') == 'true') {
        return true;
      }
    }

    return false;
  }

  /**
   * Check the payment method state.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @param string $subModule The payment method
   * @return bool
   */
  function checkSubModule($subModule = '') {
    global $xtLink;

    if ($this->subModuleIsAllowed($subModule)) {
      if ($this->subModuleIsActivated($subModule)) {
        return true;
      }
    }

    return false;
  }

  /**
   * Get the payments method credentials.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return none
   */
  function getConfFieldsBySubModules() {
    global $currency;

    foreach ($this->allowed_subpayments as $subModule) {
      if ($this->checkSubModule($subModule)) {
        $subModulePref = strtoupper(get_class($this) . '_' . $subModule . '_');
        $params = array();
        $params['enable'] = (constant($subModulePref . 'ACTIVATED') == 'true' ? true : false);
        $params['merchantId'] = constant($subModulePref . 'MID');
        $params['projectId'] = constant($subModulePref . 'PID');
        $params['secure'] = constant($subModulePref . 'SECURE');
        $params['purpose'] = constant($subModulePref . 'PURPOSE');
        $params['order'] = constant($subModulePref . 'ORDER');
        
        switch ($subModule) {
          case 'gc_giropay':
          case 'gc_ideal':
          case 'gc_eps':
          case 'gc_sofortuw':
          case 'gc_directdebit':
          case 'gc_paydirekt':
            if ($params['enable'] && $currency->code != "EUR") {
              $params['enable'] = false;
            }
            break;
          case 'gc_creditcard':
            $params['visamaster'] = constant($subModulePref . 'VISAMASTER');
            $params['amex'] = constant($subModulePref . 'AMEX');
            $params['jcb'] = constant($subModulePref . 'JCB');
            break;
        }
        $this->data[$subModule] = $params;
      }
    }
  }

  /**
   * Load the payments method information.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return none
   */
  function getSubModulesInfo() {
    // If version is less 5 
    if( version_compare(5,_SYSTEM_VERSION) == 1 )
    {
      $this->sysversion4 = 'true';
    }

    $this->data['xt_girochekout']['sysversion4'] = $this->sysversion4;
    $this->data['xt_girochekout']['payment_icon'] = _SYSTEM_BASE_URL . _SRV_WEB . _SRV_WEB_PLUGINS . 'xt_girocheckout/images/girocheckout_logo.png';
    $this->data['xt_girochekout']['titleplugin'] = 'Bezahlen mit:';
    foreach ($this->allowed_subpayments as $subModule) {

      if ($this->checkSubModule($subModule)) {
        $title = $this->aTexts[$this->lang][$subModule]['title'];
        $title[0] = strtoupper($title);
        $this->data[$subModule]['title'] = htmlspecialchars($title);
        $this->data[$subModule]['description'] = htmlspecialchars($this->aTexts[$this->lang][$subModule]['description']);
        
        if( $subModule == 'gc_creditcard' ) {

          $strLogoName = $this->getExtendedLogo();

          if( strlen($strLogoName) > 0 ) {
            $strLogoImageCredit = $strLogoName;
          }
          else {
            $strLogoImageCredit = 'gc_creditcard.jpg';
          }
          
          $this->data[$subModule]['imageUrl'] = _SYSTEM_BASE_URL . _SRV_WEB . _SRV_WEB_PLUGINS . 'xt_girocheckout/images/' . $strLogoImageCredit;
        }
        else {
          $this->data[$subModule]['imageUrl'] = _SYSTEM_BASE_URL . _SRV_WEB . _SRV_WEB_PLUGINS . 'xt_girocheckout/images/' . $subModule . '.jpg';
        }
      }
    }
  }
  

  function sortMethods($a, $b)
  {
    return ($a["order"] <= $b["order"]) ? -1 : 1;
  }

  /**
   * Check the payment name selegetSubModuleNamected.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return bool
   */
  function getSubModuleName() {
    global $_POST;
    
    if (isset($_POST['selected_payment'])) {
      $selectedPayment = $_POST['selected_payment'];
      if (strpos($selectedPayment, ':')) {
        $payments = explode(':', $selectedPayment);
        $subModule = $payments[1];
        if ($this->checkSubModule($subModule)) {
          return $subModule;
        }
      }
    } else if (isset($_SESSION['selected_payment_sub'])) {
      return $_SESSION['selected_payment_sub'];
    } else if (isset($processed_data['subpayment_code'])) {
      return $processed_data['subpayment_code'];
    }

    return false;
  }

  /**
   * Clear the plugin session.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return none
   */
  static function clearSession() {
    if (isset($_SESSION['xt_girocheckout'])) {
      unset($_SESSION['xt_girocheckout']);
    }
  }

  /**
   * Set the error message in session.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @param string $msg The error message
   * @return none
   */
  static function setErrorMsg($msg = '') {
    $_SESSION['xt_girocheckout']['error'] = $msg;
  }





  /**
   * Handle the page action.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return none
   */
  function handlePageAction($page, $action) {
    global $_POST, $filter, $xtLink, $info, $order;

    if ($page == 'checkout') {
      //Sometimes the 'payment' stage can be 'confirmation', bug in Veyton?
      if ($action == 'payment' || $action == 'confirmation') {
        
      }

      if ($action == 'confirmation' && $_SESSION['selected_payment'] == get_class($this)) {
        
      }

      if ($action == 'shipping') {
        
      }
    }
  }

  /**
   * Build the payment information.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @param array $data
   * @return bool
   */
  function build_payment_info($data) {
    return true;
  }

  /**
   * Process the payment transaction and redirect to the bank page.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @param array $processed_data
   * @return string, the link page
   */
  function pspRedirect($processed_data = array()) {
    global $xtLink, $filter, $order, $db;

    // cancel
    $tmp_link = $xtLink->_link(array(
        'page' => 'checkout',
        'paction' => 'payment',
        'params' => $xtLink->_getParams() . '&error_payments=ERROR_PAYMENT',
        'conn' => 'SSL'
    ));

    if (($subModule = $this->getSubModuleName()) !== false) {

      if ($this->checkSubModule($subModule)) {
        $customers_id = $order->customer;
        $cart = $_SESSION['cart'];
        $amount = trim(round($cart->total['plain'], 2));
        $currency = isset($processed_data['currency_code']) ? $processed_data['currency_code'] : 'EUR';
        $transaction_id = (int) $processed_data['orders_id'];
        $purpose = $this->data[$subModule]['purpose'];
        $vwz = $this->getPurpose($purpose, $order);//'Best. ' . $transaction_id;
        $amountShipping = trim(round($cart->sub_content_total['plain'], 2));
        $amountOrder = $amount - $amountShipping;
        
        $urlNotify = str_replace('&amp;', '&', $xtLink->_link(array(
                    'page' => 'callback',
                    'paction' => 'xt_girocheckout',
                    'params' => 'act=notify&pm=' . $subModule
        )));
        $urlRedirect = str_replace('&amp;', '&', $xtLink->_link(array(
                    'page' => 'callback',
                    'paction' => 'xt_girocheckout',
                    'params' => 'act=redirect&pm=' . $subModule
        )));

        // Get the shipping amount
        foreach ($order->order_total_data as $oOrderShipping) {
          if( $oOrderShipping["orders_total_key"] == 'shipping' ) {
            $amountShipping = trim(round($oOrderShipping["orders_total_price"]['plain'], 2));
          }
        }
        
        $order_query = "SELECT orders_id, customers_id FROM " . TABLE_ORDERS . " WHERE orders_id = '" . (int) $transaction_id . "'";
        $rs = $db->Execute($order_query);

        if ($rs->RecordCount() == 1) {
          $orders_id = $rs->fields['orders_id'];
        }

        switch ($subModule) {
          case 'gc_giropay': //gc_giropay
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];

            try {
              //Sends request to Girocheckout.
              $request = new GiroCheckout_SDK_Request('giropayTransaction');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                      ->addParam('projectId', $projectId)
                      ->addParam('merchantTxId', $transaction_id)
                      ->addParam('amount', $amount * 100)
                      ->addParam('currency', $currency)
                      ->addParam('purpose', $vwz)
                      ->addParam('urlRedirect', $urlRedirect)
                      ->addParam('urlNotify', $urlNotify)
                      ->addParam('sourceId', $this->getGcSource())
                      ->addParam('orderId', $transaction_id)
                      ->addParam('customerId', $customers_id)
                      ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              } else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $this->lang);
              }
            } catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang) . $e->getMessage();
            }

            break;

          case 'gc_directdebit': //gc_directdebit
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];

            try {

                $request = new GiroCheckout_SDK_Request('directDebitTransactionWithPaymentPage');
                $request->setSecret($secure);
                $request->addParam('merchantId', $merchantId)
                        ->addParam('projectId', $projectId)
                        ->addParam('merchantTxId', $transaction_id)
                        ->addParam('amount', $amount * 100)
                        ->addParam('currency', $currency)
                        ->addParam('purpose', $vwz)
                        ->addParam('urlRedirect', $urlRedirect)
                        ->addParam('urlNotify', $urlNotify)
                        ->addParam('sourceId', $this->getGcSource())
                        ->addParam('orderId', $transaction_id)
                        ->addParam('customerId', $customers_id)
                        ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              } else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $this->lang);
              }
            } catch (Exception $e) {
             $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang) . $e->getMessage();
            }
            break;

          case 'gc_creditcard': //gc_creditcard
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];
            
            try {
              $request = new GiroCheckout_SDK_Request('creditCardTransaction');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                      ->addParam('projectId', $projectId)
                      ->addParam('merchantTxId', $transaction_id)
                      ->addParam('amount', $amount * 100)
                      ->addParam('currency', $currency)
                      ->addParam('purpose', $vwz)
                      ->addParam('locale', $this->lang)
                      ->addParam('urlRedirect', $urlRedirect)
                      ->addParam('urlNotify', $urlNotify)
                      ->addParam('sourceId', $this->getGcSource())
                      ->addParam('orderId', $transaction_id)
                      ->addParam('customerId', $customers_id)
                      ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              } else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('resultPayment'), $this->lang);
              }
            } catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang);
            }

            break;

          case 'gc_eps': //gc_eps
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];

            try {
              //Sends request to Girocheckout.
              $request = new GiroCheckout_SDK_Request('epsTransaction');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                      ->addParam('projectId', $projectId)
                      ->addParam('merchantTxId', $transaction_id)
                      ->addParam('amount', $amount * 100)
                      ->addParam('currency', $currency)
                      ->addParam('purpose', $vwz)
                      ->addParam('urlRedirect', $urlRedirect)
                      ->addParam('urlNotify', $urlNotify)
                      ->addParam('sourceId', $this->getGcSource())
                      ->addParam('orderId', $transaction_id)
                      ->addParam('customerId', $customers_id)
                      ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              } else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $this->lang);
              }
            } catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang);
            }
            break;

          case 'gc_ideal': //gc_ideal
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];

            try {
              //Sends request to Girocheckout.
              $request = new GiroCheckout_SDK_Request('idealPayment');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                      ->addParam('projectId', $projectId)
                      ->addParam('merchantTxId', $transaction_id)
                      ->addParam('amount', $amount * 100)
                      ->addParam('currency', $currency)
                      ->addParam('purpose', $vwz)
                      ->addParam('urlRedirect', $urlRedirect)
                      ->addParam('urlNotify', $urlNotify)
                      ->addParam('sourceId', $this->getGcSource())
                      ->addParam('orderId', $transaction_id)
                      ->addParam('customerId', $customers_id)
                      ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              } else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('rc'), $this->lang);
              }
            } catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang);
            }
            break;
            
          case 'gc_sofortuw': 
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];
            
            try {
              $request = new GiroCheckout_SDK_Request('sofortuwTransaction');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                      ->addParam('projectId', $projectId)
                      ->addParam('merchantTxId', $transaction_id)
                      ->addParam('amount', $amount * 100)
                      ->addParam('currency', $currency)
                      ->addParam('purpose', $vwz)
                      ->addParam('urlRedirect', $urlRedirect)
                      ->addParam('urlNotify', $urlNotify)
                      ->addParam('sourceId', $this->getGcSource())
                      ->addParam('orderId', $transaction_id)
                      ->addParam('customerId', $customers_id)
                      ->submit();

              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              } else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('resultPayment'), $this->lang);
              }
            } catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang);
            }

            break;
            
          case 'gc_paydirekt': 
            $merchantId = $this->data[$subModule]['merchantId'];
            $projectId = $this->data[$subModule]['projectId'];
            $secure = $this->data[$subModule]['secure'];
            $strVorname = $order->order_data["delivery_firstname"];
            $strNachname = $order->order_data["delivery_lastname"];
            $strFirma = $order->order_data["delivery_company"];
            $strAdditionalAdr = $order->order_data["comments"];
            $strStrasse = $order->order_data["delivery_street_address"];
            $strPLZ = $order->order_data["delivery_postcode"];
            $strOrt = $order->order_data["delivery_city"];
            $strLand = $order->order_data["delivery_country_code"];
            $strMail = $order->order_data["customers_email_address"];
            
            try {
              $oCart = new GiroCheckout_SDK_Request_Cart();

              foreach ($order->order_products as $oProduct) {
                $oCart->addItem( $oProduct["products_name"], trim(round($oProduct["products_quantity"],2)), trim(round($oProduct["products_price"]["plain"], 2))*100, $oProduct["products_model"] );
              }
              
              $request = new GiroCheckout_SDK_Request('paydirektTransaction');
              $request->setSecret($secure);
              $request->addParam('merchantId', $merchantId)
                      ->addParam('projectId', $projectId)
                      ->addParam('merchantTxId', $transaction_id)
                      ->addParam('amount', $amount * 100)
                      ->addParam('currency', $currency)
                      ->addParam('purpose', $vwz)
                      ->addParam('shippingAmount', $amountShipping * 100)
                      ->addParam('shippingAddresseFirstName', $strVorname)
                      ->addParam('shippingAddresseLastName', $strNachname)
                      ->addParam('shippingCompany', $strFirma)
                      ->addParam('shippingAdditionalAddressInformation', $strAdditionalAdr)
                      ->addParam('shippingStreet',$strStrasse)
                      ->addParam('shippingZipCode', $strPLZ)
                      ->addParam('shippingCity', $strOrt)
                      ->addParam('shippingCountry', $strLand)
                      ->addParam('orderAmount', $amountOrder * 100)
                      ->addParam('customerMail', $strMail)
                      ->addParam('cart', $oCart) 
                      ->addParam('urlRedirect', $urlRedirect)
                      ->addParam('urlNotify', $urlNotify)
                      ->addParam('sourceId', $this->getGcSource())
                      ->addParam('orderId', $transaction_id)
                      ->addParam('customerId', $customers_id)
                      ->submit();
              
              if ($request->requestHasSucceeded()) {
                $request->redirectCustomerToPaymentProvider();
                exit();
              } else {
                $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage($request->getResponseParam('resultPayment'), $this->lang);
              }
            } catch (Exception $e) {
              $strError = GiroCheckout_SDK_ResponseCode_helper::getMessage(5100, $this->lang);
            }

            break;            
        }

        if (strlen($strError) > 0) {
          $erMsg = $strError;

          self::setErrorMsg($erMsg);
          $tmp_link = $xtLink->_link(array(
              'page' => 'checkout',
              'paction' => 'payment',
              'params' => $xtLink->_getParams() . '&error=' . $erMsg,
              'conn' => 'SSL'
          ));
        }
      }
    }

    return $tmp_link;
  }

  /**
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return bool
   */
  function pspSuccess() {
    return true;
  }

  /**
   * Load the plugin payments method texts.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return none
   */
  function loadText() {
    $this->aTexts['de']['merchantid'] = "Verkäufer-ID";
    $this->aTexts['en']['merchantid'] = "Merchant ID";
    $this->aTexts['de']['projectid'] = "Projekt-ID";
    $this->aTexts['en']['projectid'] = "Project ID";
    $this->aTexts['de']['passphrase'] = "Projekt-Passwort";
    $this->aTexts['en']['passphrase'] = "Project password";
    $this->aTexts['de']['info'] = "Die Daten für die Einstellungen entnehmen Sie bitte dem Projekt-Profil in Ihrem GiroCockpit (www.girocockpit.de).";
    $this->aTexts['en']['info'] = "";

    $this->aTexts['de']['gc_giropay']['title'] = "giropay - online überweisen";
    $this->aTexts['en']['gc_giropay']['title'] = "giropay - online bank transfer";
    $this->aTexts['de']['gc_giropay']['description'] = "Mit giropay zahlen Sie einfach, schnell und sicher im Online-Banking Ihrer teilnehmenden Bank oder Sparkasse. Sie werden direkt zum Online-Banking Ihrer Bank weitergeleitet, wo Sie die Überweisung durch Eingabe von PIN und TAN freigeben.";
    $this->aTexts['en']['gc_giropay']['description'] = "With giropay you pay simply, fast and secure by the online banking of your participating bank. You will be redirected to the online banking of your bank where you authorize the credit transfer.";
    $this->aTexts['de']['gc_giropay']['bank_selection'] = "Ihre Bank";
    $this->aTexts['en']['gc_giropay']['bank_selection'] = "bank";

    $this->aTexts['de']['gc_directdebit']['title'] = "Lastschrift";
    $this->aTexts['en']['gc_directdebit']['title'] = "direct debit";
    $this->aTexts['de']['gc_directdebit']['description'] = "Hiermit ermächtigen Sie uns, den fälligen Betrag von Ihrem Konto mittels Lastschrift einzuziehen.";
    $this->aTexts['en']['gc_directdebit']['description'] = "Hereby you authorize us to charge your current account with the invoiced amount by direct debit.";

    $this->aTexts['de']['gc_creditcard']['title'] = "Kreditkarte";
    $this->aTexts['en']['gc_creditcard']['title'] = "credit card";
    $this->aTexts['de']['gc_creditcard']['description'] = "Im nächsten Schritt werden Sie aufgefordert, Ihre Kreditkartendaten einzugeben. Wenn Ihre Kreditkarte für 3D-Secure (Verified by Visa bzw. MasterCard SecureCode) freigeschaltet ist, werden Sie während des Bezahlprozesses zur Sicherheitsabfrage Ihrer Bank weitergeleitet.";
    $this->aTexts['en']['gc_creditcard']['description'] = "On the following secure website, you are asked to enter your credit card data. If your credit card is enabled for 3D Secure (Verified by Visa or MasterCard Secure Code), additional information will be passed for a security check at your bank.";

    $this->aTexts['de']['gc_ideal']['title'] = "iDEAL - Online überweisen (NL)";
    $this->aTexts['en']['gc_ideal']['title'] = "iDEAL - online bank transfer (NL)";
    $this->aTexts['de']['gc_ideal']['description'] = "iDEAL ist das Online Überweisungssystem der niederländischen Banken, das auf dem etablierten Onlinebanking-Verfahren der Kreditinstitute basiert. Im folgenden müssen Sie Ihre Bank auswählen und können anschließend die Zahlung in Ihrem Online-Banking durchführen.";
    $this->aTexts['en']['gc_ideal']['description'] = "With iDEAL you pay simply, fast and secure by online banking of your bank. You will be redirected to the online banking of your bank where you authorize the credit transfer with PIN and TAN.";

    $this->aTexts['de']['gc_eps']['title'] = "eps - Online überweisung";
    $this->aTexts['en']['gc_eps']['title'] = "eps - online bank transfer (NL)";
    $this->aTexts['de']['gc_eps']['description'] = "Mit eps Online-Überweisung zahlen Sie einfach, schnell und sicher im Online-Banking Ihrer Bank. Im nächsten Schritt werden Sie direkt zum Online-Banking Ihrer Bank weitergeleitet, wo Sie die Zahlung durch Eingabe von PIN und TAN freigeben.";
    $this->aTexts['en']['gc_eps']['description'] = "With eps online transfer you pay simply, fast and secure by online banking of your bank. Your will be redirected to the online banking at your bank where you authorize the credit transfer with PIN and TAN.";
    $this->aTexts['de']['gc_eps']['bank_selection'] = "Ihre Bank";
    $this->aTexts['en']['gc_eps']['bank_selection'] = "bank";
    
    $this->aTexts['de']['gc_paydirekt']['title'] = "Paydirekt";
    $this->aTexts['en']['gc_paydirekt']['title'] = "Paydirekt";
    $this->aTexts['de']['gc_paydirekt']['description'] = "Das Online-Bezahlverfahren Ihrer Banken und Sparkassen. Sie werden auf die Webseiten von Paydirekt weiter geleitet, wo Sie sich mit Ihren persönlichen Zugangsdaten anmelden und dann die Zahlung bestätigen. Sie benötigen vorab eine Registrierung bei Paydirekt.";
    $this->aTexts['en']['gc_paydirekt']['description'] = "The online payment method of the German banks and Sparkassen. You will be redirected to the Paydirekt web site, where you will log into your personal account and authorize the payment. You require prior registration with Paydirekt. This payment method only works with German banking accounts.";
    
    $this->aTexts['de']['gc_sofortuw']['title'] = "SOFORT Überweisung";
    $this->aTexts['en']['gc_sofortuw']['title'] = "SOFORT Überweisung";
    $this->aTexts['de']['gc_sofortuw']['description'] = "Mit SOFORT Überweisung geht das online Bezahlen einfach wie nie! Sie nutzen dabei ganz bequem Ihre vertrauten Online-Banking Daten und sind so auf der sicheren Seite. Sie werden auf die SOFORT-Webseiten weiter geleitet, geben dort Ihre Bankdaten ein und lösen die Online-Überweisung aus.";
    $this->aTexts['en']['gc_sofortuw']['description'] = "Direct payment via online banking. Make payments the easy way with your own online banking details – fast and direct. You will be redirected to the web site of SOFORT, where you will enter your banking data and initiate the electronic wire transfer.";
  }

  /**
   * Returns plugin version.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return string
   */
  function getVersion() {
    return $this->version;
  }

  /**
   * Returns plugin and shop version.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return string
   */
  function getGcSource() {
    return "xt:Commerce ". _SYSTEM_VERSION . ";xt:Commerce Plugin " . $this->getVersion();
  }

  /**
   * Get the payment purpose.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return int
   */
  function getPurpose($purpose, $order) {
    $strPurpose = $purpose;

    $iCustomerNr = $order->customer;
    $strLastName = $order->order_data['delivery_lastname'];
    $strFirstName = $order->order_data['delivery_firstname'];
    $strName = $order->order_data['delivery_firstname'] . ", " . $order->order_data['delivery_lastname'];

    $strShopName = _STORE_NAME;
    $strPurpose = str_replace("{ORDERID}", $order->oID, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERID}", $iCustomerNr, $strPurpose);
    $strPurpose = str_replace("{SHOPNAME}", $strShopName, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERNAME}", $strName, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERFIRSTNAME}", $strFirstName, $strPurpose);
    $strPurpose = str_replace("{CUSTOMERLASTNAME}", $strLastName, $strPurpose);

    return utf8_encode(trim(substr($strPurpose, 0, 27)));
  }

  /**
   * Get the extended logo for creditcards.
   *
   * @author GiroSolution AG
   * @package GiroCheckout
   * @copyright Copyright (c) 2014, GiroSolution AG
   * @return string
   */
  function getExtendedLogo() {
    $subModule = 'gc_creditcard';
    $visa_msc = ($this->data[$subModule]['visamaster'] == 'true') ? 1 : 0 ;
    $amex = ($this->data[$subModule]['amex'] == 'true') ? 1 : 0;
    $jcb = ($this->data[$subModule]['jcb'] == 'true') ? 1 : 0;
            
    if (!$visa_msc && !$amex && !$jcb)
      $visa_msc = true;
    
    return GiroCheckout_SDK_Tools::getCreditCardLogoName($visa_msc, $amex, $jcb);
  }
}